#include <GL/glew.h>
#include "mesh.h"
#include <string>
#include "filereader.h"
#include "filewriter.h"
#include <QDebug>

using namespace std;

void Mesh::loadOFF(const QString &filename)
{
	FileReader stream(filename.toStdString());
	if (!stream.isOpen())
		return;
	string line;
	stream >> line;
	const bool b_color = line == "COFF";
	if (line != "OFF" && line != "COFF")
		return;

	u_int32_t nb_vtx, nb_idx, nb_edg;
	stream >> nb_vtx >> nb_idx >> nb_edg;

	vertex.reserve(nb_vtx);
	if (b_color)
		color.reserve(nb_vtx);
	for(size_t i = 0 ; i < nb_vtx ; ++i)
	{
		Vec3 p;
		stream >> p[0] >> p[1] >> p[2];
		vertex.push_back(p);
		if (b_color)
		{
			Vec3 c;
			stream >> c[0] >> c[1] >> c[2];
			c = c * 255.f;
			for(size_t j = 0 ; j < 3 ; ++j)
				c[j] = std::min(255.f, std::max(0.f, c[j]));
			color.push_back(Vec3ub(c[0], c[1], c[2]));
		}
	}
	index.reserve(nb_idx * 3);
	for(size_t i = 0 ; i < nb_idx ; ++i)
	{
		u_int32_t n;
		stream >> n;
		if (n < 2)
		{
			u_int32_t k;
			for(u_int32_t j = 0 ; j < n ; ++j)
				stream >> k;
			continue;
		}
		u_int32_t a, b;
		stream >> a >> b;
		for(u_int32_t j = 2 ; j < n ; ++j)
		{
			index.push_back(a);
			index.push_back(b);
			stream >> b;
			index.push_back(b);
		}
	}

	primitive_type = GL_TRIANGLES;

	recompute_normals();
}

void Mesh::saveOFF(const QString &filename) const
{
	FileWriter file(filename.toStdString());
	if (!file.isOpen())
		return;
	file << "OFF\n";

	u_int32_t nb_vtx(0), nb_idx(0);
	vector<const Mesh*> qmesh;
	qmesh.push_back(this);
	while(!qmesh.empty())
	{
		const Mesh *cur = qmesh.back();
		qmesh.pop_back();
		for(size_t i = 0 ; i < cur->childs.size() ; ++i)
			qmesh.push_back(cur->childs[i]);

		nb_vtx += cur->vertex.size();
		switch(cur->primitive_type)
		{
		case GL_TRIANGLES:
			nb_idx += cur->index.size() / 3;
			break;
		case GL_TRIANGLE_STRIP:
			nb_idx += cur->index.empty() ? 0 : (cur->index.size() - 2);
			break;
		}
	}

	file << nb_vtx << ' ' << nb_idx << " 0\n";

	qmesh.push_back(this);
	while(!qmesh.empty())
	{
		const Mesh *cur = qmesh.back();
		qmesh.pop_back();
		for(size_t i = 0 ; i < cur->childs.size() ; ++i)
			qmesh.push_back(cur->childs[i]);

		for(size_t i = 0 ; i < cur->vertex.size() ; ++i)
		{
			const Vec3 &v = cur->vertex[i];
			file << v.x() << ' ' << v.y() << ' ' << v.z() << '\n';
		}
	}

	qmesh.push_back(this);
	quint32 offset(0);
	while(!qmesh.empty())
	{
		const Mesh *cur = qmesh.back();
		qmesh.pop_back();
		for(size_t i = 0 ; i < cur->childs.size() ; ++i)
			qmesh.push_back(cur->childs[i]);

		switch(cur->primitive_type)
		{
		case GL_TRIANGLES:
			for(size_t i = 0 ; i < cur->index.size() ; i += 3)
				file << 3 << ' ' << (cur->index[i] + offset) << ' ' << (cur->index[i + 1] + offset) << ' ' << (cur->index[i + 2] + offset) << '\n';
			break;
		case GL_TRIANGLE_STRIP:
			for(size_t i = 0 ; i + 2 < cur->index.size() ; ++i)
				file << 3 << ' ' << (cur->index[i] + offset) << ' ' << (cur->index[i + 1] + offset) << ' ' << (cur->index[i + 2] + offset) << '\n';
			break;
		}

		offset += cur->vertex.size();
	}
}
